"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var html_entities_1 = require("html-entities");
var paperclip_utils_1 = require("paperclip-utils");
var utils_1 = require("./utils");
var entities = new html_entities_1.Html5Entities();
exports.getNativeNodePath = function (root, node) {
    var path = [];
    var current = node;
    while (current.parentNode !== root) {
        path.unshift(Array.prototype.indexOf.call(current.parentNode.childNodes, current));
        current = current.parentNode;
    }
    return path;
};
exports.createNativeNode = function (node, factory, protocol) {
    // return document.createTextNode(JSON.stringify(node, null, 2));
    if (!node) {
        return factory.createTextNode("");
    }
    try {
        switch (node.kind) {
            case "Text": {
                var text = createNativeTextNode(node, factory);
                return text;
            }
            case "Element":
                return createNativeElement(node, factory, protocol);
            case "StyleElement":
                return createNativeStyle(node, factory, protocol);
            case "Fragment":
                return createNativeFragment(node, factory, protocol);
        }
    }
    catch (e) {
        return factory.createTextNode(String(e.stack));
    }
};
var createNativeTextNode = function (node, factory) {
    return factory.createTextNode(entities.decode(node.value));
};
var createNativeStyle = function (element, factory, protocol) {
    // return factory.createTextNode(JSON.stringify(element.sheet, null, 2));
    // return factory.createTextNode(stringifyCSSSheet(element.sheet, protocol));
    var nativeElement = factory.createElement("style");
    nativeElement.textContent = paperclip_utils_1.stringifyCSSSheet(element.sheet, protocol);
    return nativeElement;
};
var createNativeElement = function (element, factory, protocol) {
    // return factory.createTextNode(JSON.stringify(element, null, 2));
    var nativeElement = factory.createElement(element.tagName);
    for (var name_1 in element.attributes) {
        var value = element.attributes[name_1];
        if (name_1 === "src" && protocol) {
            value = value.replace(/\w+:/, protocol);
        }
        nativeElement.setAttribute(name_1, value);
    }
    for (var _i = 0, _a = element.children; _i < _a.length; _i++) {
        var child = _a[_i];
        nativeElement.appendChild(exports.createNativeNode(child, factory, protocol));
    }
    // prevent redirects & vscode from asking to redirect.
    if (element.tagName === "a") {
        nativeElement.onclick = utils_1.preventDefault;
        nativeElement.onmouseup = utils_1.preventDefault;
        nativeElement.onmousedown = utils_1.preventDefault;
    }
    return nativeElement;
};
var createNativeFragment = function (fragment, factory, protocol) {
    var nativeFragment = factory.createDocumentFragment();
    for (var _i = 0, _a = fragment.children; _i < _a.length; _i++) {
        var child = _a[_i];
        nativeFragment.appendChild(exports.createNativeNode(child, factory, protocol));
    }
    return nativeFragment;
};
